#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <gmodule.h>
#include <epan/packet.h>
#include <epan/prefs.h>
#include <epan/proto.h>

#include <stdio.h>

#include <epan/emem.h>
#include <epan/dissectors/packet-tcp.h>


/* forward reference */
void proto_register_aaaa();
void proto_reg_handoff_aaaa(void);
void dissect_aaaa(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);

/* Define version if we are not building Wireshark statically */
#ifndef ENABLE_STATIC
G_MODULE_EXPORT const gchar version[] = "0.1";
#endif

#define MESSAGE_HEADER_LENGTH 28
static guint global_aaaa_port = 2900;

static guint proto_aaaa = -1;
static dissector_handle_t aaaa_handle;

static int hf_aaaa_Header_dwDataLen = -1;
static int hf_aaaa_Header_dwSeq = -1;
static int hf_aaaa_Header_Header = -1;
static int hf_aaaa_Data = -1;
static int hf_aaaa_Data_dwSeq = -1;


static hf_register_info hf[] = {
	{ &hf_aaaa_Header_dwDataLen,
	{ "Data Len",           "aaaa.Header.dwDataLen",
	FT_NONE, BASE_DEC, NULL, 0x0,
	"", HFILL }
	},
	{ &hf_aaaa_Header_dwSeq,
	{ "Header Seq",           "aaaa.Header.dwSeq",
	FT_UINT32, BASE_DEC, NULL, 0x0,
	"", HFILL }
	},
	{ &hf_aaaa_Header_Header,
	{ "Header H",           "aaaa.Header.Header",
	FT_BYTES, BASE_DEC, NULL, 0x0,
	"", HFILL }
	},
	{ &hf_aaaa_Data,
	{ "Data",           "aaaa.Header.Data",
	FT_BYTES, BASE_DEC, NULL, 0x0,
	"", HFILL }
	},
	{ &hf_aaaa_Data_dwSeq,
	{ "Data Seq",           "aaaa.Data.dwSeq",
	FT_BYTES, BASE_HEX, NULL, 0x0,
	"", HFILL }
	},
};


static gint ett_aaaa = -1;

/* Setup protocol subtree array */
static gint *ett[] = {
	&ett_aaaa,
};

static FILE *out_file = NULL;

static void dissect_aaaa_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
static guint get_aaaa_message_len(packet_info *pinfo, tvbuff_t *tvb, int offset);

/* The main dissecting routine */
static void dissect_aaaa(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	pinfo->can_desegment = TRUE;
	    tcp_dissect_pdus(tvb, pinfo, tree, TRUE, MESSAGE_HEADER_LENGTH,
	                     get_aaaa_message_len, dissect_aaaa_message);
}

/* determine PDU length of protocol foo */
static guint get_aaaa_message_len(packet_info *pinfo, tvbuff_t *tvb, int offset)
{
	guint l_gi = 4 + 4 + MESSAGE_HEADER_LENGTH + (guint) tvb_get_letohl(tvb, offset) + 4;
	return l_gi;
}

//#define FILE_OUT


#ifndef ENABLE_STATIC
G_MODULE_EXPORT void
plugin_register(void)
{
	/* register the new protocol, protocol fields, and subtrees */
	if (proto_aaaa == -1) { /* execute protocol initialization only once */
		out_file = fopen("aaaaOut.txt", "wb");
#ifndef FILE_OUT
		if(out_file)fprintf(out_file, " ");
		fclose(out_file);
		out_file = NULL;
#endif
		if(out_file)
		{
			if(out_file)fprintf(out_file, "aaaaOut.tx --- aaaa dissector\n");
		}
		proto_register_aaaa();
	}
}

G_MODULE_EXPORT void
plugin_reg_handoff(void){
	proto_reg_handoff_aaaa();
}
#endif


void
proto_register_aaaa(void)
{
	module_t *aaaa_module;

		if (proto_aaaa == -1) {
			proto_aaaa = proto_register_protocol (
				"aaaa",		/* name */
				"aaaa",		/* short name */
				"aaaa"		/* abbrev */
);
		}
		aaaa_module	= prefs_register_protocol(proto_aaaa, proto_reg_handoff_aaaa);
		proto_register_field_array(proto_aaaa, hf, array_length(hf));
		proto_register_subtree_array(ett, array_length(ett));
}

void
proto_reg_handoff_aaaa(void)
{
	static guint Initialized=FALSE;

	if (!Initialized) {
		aaaa_handle = create_dissector_handle(dissect_aaaa, proto_aaaa);
		if(out_file)fprintf(out_file, "aaaa_handle:%X\n", aaaa_handle);
		dissector_add("tcp.port", global_aaaa_port, aaaa_handle);
	}
}

static void
dissect_aaaa_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	guint32 data_len = 0;
	guint32 seq1 = 0;
	guint32 seq2 = 0;
	guint32 total_len = 0;

	guint32 packet_length = 0;

	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "aaaa");
	/* Clear out stuff in the info column */
	if(check_col(pinfo->cinfo,COL_INFO)){
		col_clear(pinfo->cinfo,COL_INFO);
	}

	data_len = tvb_get_letohl(tvb, 0);
	total_len = 4 + 4 + MESSAGE_HEADER_LENGTH + data_len + 4;

	seq1 = tvb_get_letohl(tvb, 4);
	seq2 = tvb_get_letohl(tvb, 4 + 4 + MESSAGE_HEADER_LENGTH + data_len);


	if (check_col(pinfo->cinfo, COL_INFO)) {
		col_add_fstr(pinfo->cinfo, COL_INFO, " l:%u tl:%u s1:%u  s2:%u"
		, data_len
		, total_len
		, seq1
		, seq2);
	}

	if (tree) {
		proto_item* ti = NULL;
		proto_tree* new_tree = NULL;
		proto_item* tiData =NULL;
		proto_tree* data_tree = NULL;
		guint offset = 0;

		ti = proto_tree_add_item(tree, proto_aaaa, tvb, 0, total_len, TRUE);
//		ti = proto_tree_add_item(tree, proto_aaaa, tvb, 0, 4 + 4 + MESSAGE_HEADER_LENGTH, TRUE);
		proto_item_append_text(ti, " l:%u tl:%u s1:%u  s2:%u"
		, data_len
		, total_len
		, seq1
		, seq2);
		new_tree = proto_item_add_subtree(ti, ett_aaaa);
		proto_tree_add_item(new_tree, hf_aaaa_Header_dwDataLen, tvb, offset, 4, TRUE); offset += 4;
		proto_tree_add_item(new_tree, hf_aaaa_Header_dwSeq, tvb, offset, 4, TRUE); offset += 4;
		proto_tree_add_item(new_tree, hf_aaaa_Header_Header, tvb, offset, MESSAGE_HEADER_LENGTH, TRUE); offset += MESSAGE_HEADER_LENGTH;


		tiData = proto_tree_add_item(new_tree, proto_aaaa, tvb, offset, data_len + 4, TRUE);
		proto_item_append_text(tiData, ", Data - %u bytes", data_len + 4);

		data_tree = proto_item_add_subtree(tiData, ett_aaaa);
		proto_tree_add_item(data_tree, hf_aaaa_Data, tvb, offset, data_len, TRUE); offset += data_len;
		proto_tree_add_item(data_tree, hf_aaaa_Data_dwSeq, tvb, offset, 4, TRUE); offset += 4;

	}
}
